# coding: utf-8
from __future__ import unicode_literals

import django
from .registry import Registry

__version__ = '0.1-alpha.1'

__all__ = ['default_registry', 'register']

default_registry = Registry()
register = default_registry.register

default_app_config = 'smoked.apps.SmokedApp'

if django.VERSION[1] < 7:
    # Dj<1.7 doesn't have App registry
    from .loader import load_test_module
    load_test_module()



# coding: utf-8
from __future__ import unicode_literals

from django.apps import AppConfig


class SmokedApp(AppConfig):
    name = 'smoked'

    def ready(self):
        from smoked.loader import load_test_module
        load_test_module()


# coding: utf-8
from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

try:
    from importlib import import_module
except ImportError:
    # Py2.6,Py3.0 && Dj<1.9
    from django.utils.importlib import import_module


def load_test_module():
    """
    Import test module and trigger registration of tests. Test module is
    defined in `SMOKE_TESTS` setting.
    """

    test_module = getattr(settings, 'SMOKE_TESTS')

    if not test_module:
        raise ImproperlyConfigured('Missing SMOKE_TESTS in settings.')

    try:
        import_module(test_module)
    except ImportError as e:
        msg = "Can't import '{0}' module. Exception: {1}"
        raise ImproperlyConfigured(msg.format(test_module, e))



# coding: utf-8
from __future__ import absolute_import, unicode_literals
from optparse import make_option

import time
from django import VERSION
from django.core.management.base import NoArgsCommand
from smoked import default_registry

from smoked.runner import run_tests

stats_msg = """
Results
=======
Total: {total}
Success: {success}
Failure: {failure}
--------
Time: {time:.1f}s
"""


class Command(NoArgsCommand):
    help = 'Run all registered smoke tests'

    option_list = NoArgsCommand.option_list + (
        make_option(
            '-n', '--dry-run', dest='dry_run',
            action='store_true', default=False,
            help="Only collect test, don't execute them."),
        )

    def handle_noargs(self, **options):
        verbosity = int(options.get('verbosity', 1))
        start_time = time.time()

        if options.get('dry_run'):
            count = len(default_registry.tests)
            if verbosity:
                self.log('{0} smoke test(s) could be run'.format(count))
            else:
                self.log(str(count))
            return

        success = failure = 0
        for result in run_tests():
            positive = 'error' not in result
            if positive:
                success += 1
            else:
                failure += 1

            if verbosity > 1:
                output = 'Success' if positive else 'Fail!'
                self.log('{0}... {1}'.format(result['name'], output))

                if not positive:
                    self.log(str(result['error']))
            else:
                output = '.' if positive else 'F'
                self.log(output, ending='')

        stats = {
            'total': success + failure,
            'success': success,
            'failure': failure,
            'time': time.time() - start_time
        }
        if verbosity:
            self.log(stats_msg.format(**stats))
        else:
            self.log('')  # print out new line after dots

    def log(self, msg='', ending='\n'):
        # Backward compatability with Dj1.4
        if VERSION[1] == 4:
            self.stdout.write(msg + ending)
        else:
            self.stdout.write(msg, ending=ending)



# coding: utf-8
from __future__ import unicode_literals

from django.template.response import TemplateResponse
from smoked.runner import run_tests


def smoked_results(request):
    results = list(run_tests())

    total = len(results)
    success = sum(1 for item in results if 'error' not in item)

    ctx = {
        'results': results,
        'total': total,
        'success': success,
        'failure': total - success,
    }

    return TemplateResponse(request, 'smoked/results.html', ctx)



from smoked import default_registry


def run_tests(registry=None):
    if not registry:
        registry = default_registry

    for test in registry.tests:
        output = {
            'name': test.name,
            'description': test.description,
        }

        params = test.params or {}
        try:
            test.func(**params)
        except Exception as e:
            output['error'] = e

        yield output



# coding: utf-8
from __future__ import unicode_literals
from collections import namedtuple

SmokeTest = namedtuple('SmokeTest', 'func params name description')


class Registry(object):
    def __init__(self):
        self._registry = []

    @property
    def tests(self):
        """ List of all registered tests """
        return self._registry

    def register(self, func=None, **kwargs):
        """
        Entry point for registering smoke tests. Accepts dotted path to smoke
        test, testing func or it can act as an decorator.

        Usage:
        1. Lazily loaded SmokeTest class
        register('module.SmokeTest')

        2. Registering ad-hoc function
        register(smoke_test, name='Smoke Test', description='Help text')

        3. Simple decorator, taking description from doctest when available
        @register
        def smoke_test():
            " Description from doctest "
            pass

        4. Explicitly set name and description of decorated smoke test
        @register(name='Smoke Test', description='Explaining message')
        def smoke_test():
            pass
        """

        def wrap(inner_func):
            self._register(func=inner_func, **kwargs)
            return inner_func

        if not func:
            return wrap

        if callable(func) and func.__name__ != '<lambda>':
            kwargs.setdefault('name', func.__name__)
            kwargs.setdefault('description', func.__doc__)

        self._register(func=func, **kwargs)
        return func

    def _register(self, func, params=None, name=None, description=None):
        """
        The *real* register method responsible for adding smoke test
        into registry.
        """
        self._registry.append(SmokeTest(func, params, name, description))



# coding: utf-8
from __future__ import unicode_literals

import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

from smoked import __version__

test_requirements = [
    'pytest',
    'pytest-django',
    'pytest-mock',
]

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        sys.exit(pytest.main(self.test_args))

setup(
    name='django-smoked',
    version=__version__,
    license='MIT',

    author='Tomáš Ehrlich',
    author_email='tomas.ehrlich@gmail.com',

    description='Smoke tests framework for Django Web Framework',
    long_description=open('README.md').read(),
    url='https://github.com/djentlemen/django-smoked',

    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'django',
    ],

    cmdclass={'test': PyTest},
    tests_require=test_requirements,
    extras_require={
        'tests': test_requirements,
        'qa': [
            'pytest-cov',
            'coveralls'
        ],
    },

    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ),
)



# coding: utf-8
from __future__ import unicode_literals

from django.utils.six.moves.urllib.request import urlopen


def url_available(url=None, expected_code=200):
    """ Check availability (HTTP response code) of single resource """
    assert urlopen(url).getcode() == expected_code
